<?php

declare(strict_types=1);

namespace App\Controller;

use App\DTO\Edge\Response\Mapper\WeatherAtTimePointResponseMapper;
use App\DTO\Edge\Response\Mapper\WeatherAtTimeResponseMapper;
use App\DTO\Edge\Response\WeatherAtTimeResponse;
use App\Services\GeoPointResolverService;
use App\Services\WeatherStorageReceiverInterface;
use Nelmio\ApiDocBundle\Attribute\Model;
use OpenApi\Attributes as OA;
use Psr\Log\LoggerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;

class ForecastController extends AbstractController
{
    public function __construct(
        protected readonly string $forecastForDays,
        protected readonly WeatherStorageReceiverInterface $weatherStorageReceiver,
        protected readonly WeatherAtTimePointResponseMapper $weatherAtTimePointResponseMapper,
        protected readonly WeatherAtTimeResponseMapper $weatherAtTimeResponseMapper,
        protected readonly GeoPointResolverService $geoPointResolverService,
        protected readonly LoggerInterface $logger,
    ) {
    }

    #[OA\Response(
        response: 200,
        description: 'Returns weather forecast for city',
        content: new OA\JsonContent(
            type: 'array',
            items: new OA\Items(ref: new Model(type: WeatherAtTimeResponse::class, groups: ['full']))
        )
    )]
    #[OA\Response(
        response: 404,
        description: 'City or data not found',
    )]
    #[OA\Response(
        response: 429,
        description: 'Please slow down... Too much requests',
    )]
    #[OA\Parameter(
        name: 'cityName',
        description: 'Name of searched city',
        in: 'path',
    )]
    #[Route('/api/v1/forecast/{cityName}', name: 'getForecast', methods: ['GET'])]
    public function getForecast(string $cityName): JsonResponse
    {
        $cityGeo = $this->geoPointResolverService->getGeoPointFromCityName($cityName);

        if ($cityGeo === null) {
            $this->logger->warning('City location not found for {cityName}', ['cityName' => $cityName]);
            return $this->json('{}', Response::HTTP_NOT_FOUND);
        }

        $startDate = new \DateTime();
        $startDate->add(new \DateInterval('P1D'));

        $endDate = \DateTime::createFromInterface($startDate);
        $endDate->add(new \DateInterval($this->forecastForDays));


        $receivedData = $this->weatherStorageReceiver->receiveWeatherForPointByDay(
            $cityGeo,
            $startDate,
            $endDate
        );

        if (count($receivedData) === 0) {
            $this->logger->notice('Forecast not found for {cityName}', ['cityName' => $cityName, 'cityGeo' => $cityGeo]);
            return $this->json('{}', Response::HTTP_NOT_FOUND);
        }

        return $this->json(
            $this->weatherAtTimeResponseMapper->mapResponse(
                $cityName,
                $this->weatherAtTimePointResponseMapper->fromWeatherAtTimePointsDateAsYYYYmmdd($receivedData)
            )
        );
    }
}
