<?php

declare(strict_types=1);

namespace App\Services\Remote\Mapper;

use App\DTO\Factory\WeatherAtTimePointFactory;
use App\DTO\WeatherAtTimePoint;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Validator\ValidatorInterface;

/**
 * Example data:
 *
 * {
 *  "latitude": 50.08,
 *  "longitude": 14.439999,
 *  "generationtime_ms": 0.05459785461425781,
 *  "utc_offset_seconds": 3600,
 *  "timezone": "Europe/Prague",
 *  "timezone_abbreviation": "GMT+1",
 *  "elevation": 219,
 *  "daily_units": {
 *    "time": "iso8601",
 *    "temperature_2m_max": "°C",
 *    "temperature_2m_min": "°C"
 *  },
 *  "daily": {
 *    "time": [
 *      "2025-01-26",
 *      "2025-01-27",
 *      "2025-01-28",
 *      "2025-01-29",
 *      "2025-01-30",
 *      "2025-01-31",
 *      "2025-02-01"
 *    ],
 *    "temperature_2m_max": [
 *      9.1,
 *      8.3,
 *      7.6,
 *      7.6,
 *      7.2,
 *      2.7,
 *      2.5
 *    ],
 *    "temperature_2m_min": [
 *      3.1,
 *      2.3,
 *      4.6,
 *      3.6,
 *      1.3,
 *      -0.6,
 *      -0.8
 *    ]
 *  }
 * }
 */
class OpenMeteoMapper
{
    protected const TIME_PART_KEY = 'time';

    public function __construct(
        protected readonly WeatherAtTimePointFactory $weatherAtTimePointFactory,
        protected readonly ValidatorInterface $validator,
    ) {
    }

    /**
     * @param array<string, mixed> $responseJson
     * @return array<WeatherAtTimePoint>
     */
    public function mapResponseIntoTimePoints(
        array $responseJson,
        string $requestedPart,
        string $maxKey = 'temperature_2m_max',
        string $minKey = 'temperature_2m_min',
        ?string $valueKey = null
    ): array {
        $timePoints = [];

        $this->validateData($responseJson, $requestedPart, $minKey, $maxKey, $valueKey);

        foreach ($responseJson[$requestedPart]["time"] as $key => $value) {
            $timePoints[] = $this->weatherAtTimePointFactory->createFromUntrustedWithDateYYYYmmdd(
                $value,
                $responseJson[$requestedPart][$minKey][$key],
                $responseJson[$requestedPart][$maxKey][$key],
                $responseJson[$requestedPart][$valueKey] ?? null
            );
        }

        return $timePoints;
    }

    /**
     * @param array<string, mixed> $responseJson
     */
    protected function validateData(array $responseJson, string $requestedPart, string $minKey, string $maxKey, ?string $value): void
    {
        $unitsKey = $requestedPart . '_units';

        $constraints = new Assert\Collection([
            $unitsKey => new Assert\Collection([
                static::TIME_PART_KEY => new Assert\EqualTo('iso8601'),
                $maxKey => new Assert\EqualTo('°C'),
                $minKey => new Assert\EqualTo('°C'),
                $value => new Assert\Optional(new Assert\EqualTo('°C'))
            ], allowExtraFields: true),
            $requestedPart => new Assert\Collection([
                static::TIME_PART_KEY => new Assert\All([
                    new Assert\Date()
                ]),
                $maxKey => new Assert\All([
                    new Assert\Type('float')
                ]),
                $minKey => new Assert\All([
                    new Assert\Type('float')
                ]),
                $value => new Assert\Optional(
                    new Assert\All([
                        new Assert\Type('float')
                    ])
                )
            ], allowExtraFields: true),
        ], allowExtraFields: true);

        $errors = $this->validator->validate($responseJson, $constraints);

        if ($errors->count() === 0) {
            return;
        }

        $errorMessages = [];

        foreach ($errors as $error) {
            $errorMessages[] = $error->getPropertyPath() . ' ' . $error->getMessage();
        }

        throw new \InvalidArgumentException("Response validation error . " . implode(',', $errorMessages));
    }
}
