<?php

declare(strict_types=1);

namespace App\Services\Remote;

use App\DTO\GEOPoint;
use App\DTO\WeatherAtTimePoint;
use App\Services\Remote\Mapper\OpenMeteoMapper;
use App\Services\WeatherProviderInterface;
use App\Utils\Weather\WeatherInterval;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class OpenMeteoService implements WeatherProviderInterface
{
    protected const DAILY_REQUEST = 'daily';

    protected const TEMP_MAX = 'temperature_2m_max';

    protected const TEMP_MIN = 'temperature_2m_min';

    public function __construct(
        protected readonly HttpClientInterface $httpClient,
        protected readonly OpenMeteoMapper $openMeteoMapper,
        protected string $meteoApiUrl,
    ) {
    }

    /**
     * @return array<WeatherAtTimePoint>
     */
    public function fetchWeather(GEOPoint $geoPoint, WeatherInterval $weatherInterval, \DateTimeInterface $fromDate, \DateTimeInterface $toDate): array
    {
        return match ($weatherInterval) {
            WeatherInterval::DAILY => $this->fetchDailyWeather($geoPoint, $fromDate, $toDate),
        };
    }

    /**
     * @return array<WeatherAtTimePoint>
     */
    protected function fetchDailyWeather(GEOPoint $geoPoint, \DateTimeInterface $fromDate, \DateTimeInterface $toDate): array
    {
        $queryArgs = $this->buildQueryArgs($geoPoint, $fromDate, $toDate, static::DAILY_REQUEST, [static::TEMP_MAX, static::TEMP_MIN]);
        $response = $this->httpClient->request(
            'GET',
            $this->meteoApiUrl,
            ['query' => $queryArgs]
        );

        $responseJson = $response->toArray();

        return $this->openMeteoMapper->mapResponseIntoTimePoints($responseJson, static::DAILY_REQUEST);
    }

    /**
     * @param array<int|string, string> $needValues
     * @return array<string, string>
     */
    protected function buildQueryArgs(GEOPoint $geoPoint, \DateTimeInterface $fromDate, \DateTimeInterface $toDate, string $endPoint, array $needValues): array
    {
        return [
            'latitude' => (string)round($geoPoint->latitude, 2),
            'longitude' => (string)round($geoPoint->longitude, 2),
            $endPoint => implode(',', $needValues),
            'timezone' => 'UTC',
            'start_date' => $fromDate->format('Y-m-d'),
            'end_date' => $toDate->format('Y-m-d'),
        ];
    }
}
