<?php

declare(strict_types=1);

namespace App\Services\Storage;

use App\DTO\GEOPoint;
use App\DTO\WeatherAtTimePoint;
use App\Services\WeatherStorageReceiverInterface;
use App\Services\WeatherStorageStoreInterface;
use DateTimeInterface;
use JMS\Serializer\SerializerInterface;
use Psr\Log\LoggerInterface;

class RedisStorageStore implements WeatherStorageStoreInterface, WeatherStorageReceiverInterface
{
    protected const BY_DAY = 'DAY';

    public function __construct(
        protected readonly \Redis $redis,
        protected readonly LoggerInterface $logger,
        protected readonly SerializerInterface $serializer
    ) {
    }

    /**
     * @param array<WeatherAtTimePoint> $weatherAtTimePoints
     */
    public function storeWeatherForPointByDay(GEOPoint $point, array $weatherAtTimePoints): void
    {
        foreach ($weatherAtTimePoints as $weatherAtTimePoint) {
            $this->logger->debug("Storing new forcast for $point at date " . $weatherAtTimePoint->date->format('Y-m-d'), [
                'forecast' => $weatherAtTimePoint
            ]);

            $key = $this->getKeyFromWheatherAtTimePoint(
                \DateTime::createFromInterface($weatherAtTimePoint->date)->setTime(0, 0),
                $point,
                static::BY_DAY
            );

            if ($this->redis->set($key, $this->serializer->serialize($weatherAtTimePoint, 'json')) === false) {
                throw new \RuntimeException("Can't store data into redis");
            }
        }
    }

    public function receiveWeatherForPointByDay(GEOPoint $point, DateTimeInterface $startDate, DateTimeInterface $endDate): array
    {
        $weatherData = [];

        if ($startDate > $endDate) {
            throw new \LogicException('Start date cannot be before end date');
        }

        $this->logger->debug("Retrieving forecast for city by $point");

        $date = \DateTime::createFromInterface($startDate)->setTime(0, 0);
        $endDate = \DateTime::createFromInterface($endDate)->setTime(0, 0);
        do {
            $key = $this->getKeyFromWheatherAtTimePoint($date, $point, self::BY_DAY);
            $data = $this->redis->get($key);
            if ($data !== false) {
                $weatherData[] = $this->serializer->deserialize($data, WeatherAtTimePoint::class, 'json');
            }
            $date = $date->add(new \DateInterval('P1D'));
        } while ($date <= $endDate);

        return $weatherData;
    }

    protected function getKeyFromWheatherAtTimePoint(DateTimeInterface $weatherItemDate, GEOPoint $point, string $type): string
    {
        return implode('|', [
            'schema_version' => WeatherAtTimePoint::SCHEMA_VERSION,
            'type' => $type,
            'lat' => $point->latitude,
            'lon' => $point->longitude,
            'date' => $weatherItemDate->format(DateTimeInterface::ISO8601_EXPANDED),
        ]);
    }
}
